program Demo;

{$APPTYPE CONSOLE}

uses
  SysUtils, JNI, JavaBridge;

{$R *.res}

type
  TTest = class
  private
    FBridge: TJavaBridge;
    FCls: JClass;
    FObj: JObject;
    FInit: JMethodID;
    FHello: JMethodID;
    FValue: JFieldID;
    function GetValue: Integer;
    procedure SetValue(Value: Integer);
  public
    constructor Create(Bridge: TJavaBridge);
    procedure Hello(const Text: string);
    property Value: Integer read GetValue write SetValue;
  end;

constructor TTest.Create(Bridge: TJavaBridge);
begin
  inherited Create;
  FBridge := Bridge;
  FCls := FBridge.FindClass('test/Test');
  FInit := FBridge.GetMethodID(FCls, '<init>', '()V');
  FHello := FBridge.GetMethodID(FCls, 'hello', '(Ljava/lang/String;)V');
  FValue := FBridge.GetFieldID(FCls, 'value', 'I');
  FObj := FBridge.NewObjectA(FCls, FInit, nil);
end;

function TTest.GetValue: Integer;
begin
  Result := FBridge.GetIntField(FObj, FValue);
end;

procedure TTest.SetValue(Value: Integer);
begin
  FBridge.SetIntField(FObj, FValue, Value);
end;

procedure TTest.Hello(const Text: string);
begin
  FBridge.CallNonvirtualVoidMethodA(FObj, FCls, FHello, PJValue(FBridge.CreateJArguments([FBridge.CreateJString(Text)])));
end;

var
  Bridge: TJavaBridge;
  MyTest: TTest;
begin
  try
    Bridge := TJavaBridge.Create;
    try
      MyTest := TTest.Create(Bridge);
      try
        WriteLn('Value = ' + IntToStr(MyTest.Value));
        MyTest.Value := 10;
        WriteLn('Value = ' + IntToStr(MyTest.Value));
        MyTest.Hello('Hello, world!');
      finally
        MyTest.Free;
      end;
    finally
      Bridge.Free;
    end;
  except
    on E: Exception do
      WriteLn(E.ClassName, ': ', E.Message);
  end;
end.