(*
 * Copyright (c) 1996, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *)

(*
 * We used part of Netscape's Java Runtime Interface (JRI) as the starting
 * point of our design and implementation.
 *)

(******************************************************************************
 * Java Runtime Interface
 * Copyright (c) 1996 Netscape Communications Corporation. All rights reserved.
 *****************************************************************************)

unit JNI;

interface

{$ifdef CONDITIONALEXPRESSIONS}
  {$if CompilerVersion >= 14}
    {$define D6PLUS} // Delphi 6 or higher
  {$ifend}

  {$if CompilerVersion >= 15}
    {$define D7PLUS} // Delphi 7 or higher
  {$ifend}
{$else}
  {$define MSWINDOWS}
{$endif}

{$ifdef D7PLUS}
  {$WARN UNSAFE_TYPE OFF}
{$endif}

type
{$ifndef D6PLUS}
  PPointer = ^Pointer;
{$endif}

  JInt = Integer;
  JLong = Int64;
  JByte = ShortInt;

  JBoolean = Boolean;
  JChar = WideChar;
  JShort = SmallInt;
  JFloat = Single;
  JDouble = Double;
  JSize = JInt;

  JObject = Pointer;
  JClass = JObject;
  JThrowable = JObject;
  JString = JObject;
  JArray = JObject;
  JBooleanArray = JArray;
  JByteArray = JArray;
  JCharArray = JArray;
  JShortArray = JArray;
  JIntArray = JArray;
  JLongArray = JArray;
  JFloatArray = JArray;
  JDoubleArray = JArray;
  JObjectArray = JArray;

  JWeak = JObject;

  JValue = packed record
    case Integer of
      0: (Z: JBoolean);
      1: (B: JByte);
      2: (C: JChar);
      3: (S: JShort);
      4: (I: JInt);
      5: (J: JLong);
      6: (F: JFloat);
      7: (D: JDouble);
      8: (L: JObject);
  end;

  JFieldID = Pointer;
  JMethodID = Pointer;
  
  PJInt = ^JInt;
  PJLong = ^JLong;
  PJByte = ^JByte;
  PJBoolean = ^JBoolean;
  PJChar = ^JChar;
  PJShort = ^JShort;
  PJFloat = ^JFloat;
  PJDouble = ^JDouble;
  PJValue = ^JValue;

  VaList = Pointer;

  // Return values from jobjectRefType
  JObjectRefType = (
    JNIInvalidRefType {$IFDEF D6PLUS} = 0 {$ENDIF D6PLUS},
    JNILocalRefType {$IFDEF D6PLUS} = 1 {$ENDIF D6PLUS},
    JNIGlobalRefType {$IFDEF D6PLUS} = 2 {$ENDIF D6PLUS},
    JNIWeakGlobalRefType {$IFDEF D6PLUS} = 3 {$ENDIF D6PLUS}
  );

const
  JNI_FALSE = 0;
  JNI_TRUE = 1;

  // possible return values for JNI functions.
const
  JNI_OK = 0; // success
  JNI_ERR = -1; // unknown error
  JNI_EDETACHED = -2; // thread detached from the VM
  JNI_EVERSION = -3; // JNI version error
  JNI_ENOMEM = -4; // not enough memory
  JNI_EEXIST = -5; // VM already created
  JNI_EINVAL = -6; // invalid arguments

  // used in ReleaseScalarArrayElements
  JNI_COMMIT = 1;
  JNI_ABORT = 2;

type
  // used in RegisterNatives to describe native method name, signature, and function pointer
  JNINativeMethod = packed record
    Name: PAnsiChar;
    Signature: PAnsiChar;
    FnPtr: Pointer;
  end;

  PJNINativeMethod = ^JNINativeMethod;

  JavaVM = ^JNIInvokeInterface_;
  PJavaVM = ^JavaVM;
  
  JNIEnv = ^JNINativeInterface_;
  PJNIEnv = ^JNIEnv;

  JNINativeInterface_ = packed record
    reserved0: Pointer;
    reserved1: Pointer;
    reserved2: Pointer;
    reserved3: Pointer;

    GetVersion: function(Env: PJNIEnv): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    DefineClass: function(Env: PJNIEnv; const Name: PAnsiChar; Loader: JObject; const Buf: PJByte; Len: JSize): JClass; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    FindClass: function(Env: PJNIEnv; const Name: PAnsiChar): JClass; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    FromReflectedMethod: function(Env: PJNIEnv; Method: JObject): JMethodID; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    FromReflectedField: function(Env: PJNIEnv; Field: JObject): JFieldID; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    ToReflectedMethod: function(Env: PJNIEnv; Cls: JClass; MethodID: JMethodID; IsStatic: JBoolean): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetSuperclass: function(Env: PJNIEnv; Sub: JClass): JClass; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    IsAssignableFrom: function(Env: PJNIEnv; Sub, Sup: JClass): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    ToReflectedField: function(Env: PJNIEnv; Cls: JClass; FieldID: JFieldID; IsStatic: JBoolean): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    Throw: function(Env: PJNIEnv; Obj: JThrowable): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ThrowNew: function(Env: PJNIEnv; Clazz: JClass; const Msg: PAnsiChar): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ExceptionOccurred: function(Env: PJNIEnv): JThrowable; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ExceptionDescribe: procedure(Env: PJNIEnv); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ExceptionClear: procedure(Env: PJNIEnv); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    FatalError: procedure(Env: PJNIEnv; const Msg: PAnsiChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    PushLocalFrame: function(Env: PJNIEnv; Capacity: JInt): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    PopLocalFrame: function(Env: PJNIEnv; Result: JObject): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    NewGlobalRef: function(Env: PJNIEnv; LObj: JObject): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    DeleteGlobalRef: procedure(Env: PJNIEnv; GRef: JObject); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    DeleteLocalRef: procedure(Env: PJNIEnv; Obj: JObject); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    IsSameObject: function(Env: PJNIEnv; Obj1, Obj2: JObject): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewLocalRef: function(Env: PJNIEnv; Ref: JObject): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    EnsureLocalCapacity: function(Env: PJNIEnv; Capacity: JInt): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    AllocObject: function(Env: PJNIEnv; Clazz: JClass): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewObject: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewObjectV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewObjectA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetObjectClass: function(Env: PJNIEnv; Obj: JObject): JClass; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    IsInstanceOf: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetMethodID: function(Env: PJNIEnv; Clazz: JClass; const Name, Sig: PAnsiChar): JMethodID; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallObjectMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallObjectMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallObjectMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallBooleanMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallBooleanMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallBooleanMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallByteMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallByteMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallByteMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallCharMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallCharMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallCharMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallShortMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallShortMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallShortMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallIntMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallIntMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallIntMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallLongMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallLongMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallLongMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallFloatMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallFloatMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallFloatMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallDoubleMethod: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallDoubleMethodV: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallDoubleMethodA: function(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallVoidMethod: procedure(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID { ... } ); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallVoidMethodV: procedure(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; Args: VaList); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallVoidMethodA: procedure(Env: PJNIEnv; Obj: JObject; MethodID: JMethodID; const Args: PJValue); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualObjectMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualObjectMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualObjectMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualBooleanMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualBooleanMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualBooleanMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualByteMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualByteMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualByteMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualCharMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualCharMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualCharMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualShortMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualShortMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualShortMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualIntMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualIntMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualIntMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualLongMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualLongMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualLongMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualFloatMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualFloatMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualFloatMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualDoubleMethod: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualDoubleMethodV: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualDoubleMethodA: function(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallNonvirtualVoidMethod: procedure(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID { ... } ); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualVoidMethodV: procedure(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; Args: VaList); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallNonvirtualVoidMethodA: procedure(Env: PJNIEnv; Obj: JObject; Clazz: JClass; MethodID: JMethodID; const Args: PJValue); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetFieldID: function(Env: PJNIEnv; Clazz: JClass; const Name, Sig: PAnsiChar): JFieldID; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetObjectField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetBooleanField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetByteField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetCharField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetShortField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetIntField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetLongField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetFloatField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetDoubleField: function(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    SetObjectField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JObject); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetBooleanField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JBoolean); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetByteField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JByte); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetCharField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetShortField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JShort); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetIntField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetLongField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JLong); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetFloatField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JFloat); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetDoubleField: procedure(Env: PJNIEnv; Obj: JObject; FieldID: JFieldID; Val: JDouble); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetStaticMethodID: function(Env: PJNIEnv; Clazz: JClass; const Name, Sig: PAnsiChar): JMethodID; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticObjectMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticObjectMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticObjectMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticBooleanMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticBooleanMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticBooleanMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticByteMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticByteMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticByteMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticCharMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticCharMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticCharMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticShortMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticShortMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticShortMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticIntMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticIntMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticIntMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticLongMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticLongMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticLongMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticFloatMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticFloatMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticFloatMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticDoubleMethod: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID { ... } ): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticDoubleMethodV: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; Args: VaList): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticDoubleMethodA: function(Env: PJNIEnv; Clazz: JClass; MethodID: JMethodID; const Args: PJValue): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    CallStaticVoidMethod: procedure(Env: PJNIEnv; Cls: JClass; MethodID: JMethodID { ... } ); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticVoidMethodV: procedure(Env: PJNIEnv; Cls: JClass; MethodID: JMethodID; Args: VaList); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    CallStaticVoidMethodA: procedure(Env: PJNIEnv; Cls: JClass; MethodID: JMethodID; const Args: PJValue); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetStaticFieldID: function(Env: PJNIEnv; Clazz: JClass; const Name, Sig: PAnsiChar): JFieldID; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetStaticObjectField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStaticBooleanField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStaticByteField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStaticCharField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStaticShortField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStaticIntField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStaticLongField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStaticFloatField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStaticDoubleField: function(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID): JDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    SetStaticObjectField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JObject); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetStaticBooleanField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JBoolean); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetStaticByteField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JByte); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetStaticCharField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetStaticShortField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JShort); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetStaticIntField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetStaticLongField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JLong); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetStaticFloatField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JFloat); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetStaticDoubleField: procedure(Env: PJNIEnv; Clazz: JClass; FieldID: JFieldID; Value: JDouble); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    NewString: function(Env: PJNIEnv; const Unicode: PJChar; Len: JSize): JString; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStringLength: function(Env: PJNIEnv; Str: JString): JSize; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetStringChars: function(Env: PJNIEnv; Str: JString; var IsCopy: JBoolean): PJChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseStringChars: procedure(Env: PJNIEnv; Str: JString; const Chars: PJChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    NewStringUTF: function(Env: PJNIEnv; const Utf: PAnsiChar): JString; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStringUTFLength: function(Env: PJNIEnv; Str: JString): JSize; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetStringUTFChars: function(Env: PJNIEnv; Str: JString; var IsCopy: JBoolean): PAnsiChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseStringUTFChars: procedure(Env: PJNIEnv; Str: JString; const Chars: PAnsiChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetArrayLength: function(Env: PJNIEnv; Array_: JArray): JSize; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    NewObjectArray: function(Env: PJNIEnv; Len: JSize; Clazz: JClass; Init: JObject): JObjectArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetObjectArrayElement: function(Env: PJNIEnv; Array_: JObjectArray; Index: JSize): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetObjectArrayElement: procedure(Env: PJNIEnv; Array_: JObjectArray; Index: JSize; Val: JObject); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    NewBooleanArray: function(Env: PJNIEnv; Len: JSize): JBooleanArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewByteArray: function(Env: PJNIEnv; Len: JSize): JByteArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewCharArray: function(Env: PJNIEnv; Len: JSize): JCharArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewShortArray: function(Env: PJNIEnv; Len: JSize): JShortArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewIntArray: function(Env: PJNIEnv; Len: JSize): JIntArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewLongArray: function(Env: PJNIEnv; Len: JSize): JLongArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewFloatArray: function(Env: PJNIEnv; Len: JSize): JFloatArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    NewDoubleArray: function(Env: PJNIEnv; Len: JSize): JDoubleArray; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetBooleanArrayElements: function(Env: PJNIEnv; Array_: JBooleanArray; var IsCopy: JBoolean): PJBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetByteArrayElements: function(Env: PJNIEnv; Array_: JByteArray; var IsCopy: JBoolean): PJByte; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetCharArrayElements: function(Env: PJNIEnv; Array_: JCharArray; var IsCopy: JBoolean): PJChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetShortArrayElements: function(Env: PJNIEnv; Array_: JShortArray; var IsCopy: JBoolean): PJShort; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetIntArrayElements: function(Env: PJNIEnv; Array_: JIntArray; var IsCopy: JBoolean): PJInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetLongArrayElements: function(Env: PJNIEnv; Array_: JLongArray; var IsCopy: JBoolean): PJLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetFloatArrayElements: function(Env: PJNIEnv; Array_: JFloatArray; var IsCopy: JBoolean): PJFloat; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetDoubleArrayElements: function(Env: PJNIEnv; Array_: JDoubleArray; var IsCopy: JBoolean): PJDouble; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    ReleaseBooleanArrayElements: procedure(Env: PJNIEnv; Array_: JBooleanArray; Elems: PJBoolean; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseByteArrayElements: procedure(Env: PJNIEnv; Array_: JByteArray; Elems: PJByte; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseCharArrayElements: procedure(Env: PJNIEnv; Array_: JCharArray; Elems: PJChar; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseShortArrayElements: procedure(Env: PJNIEnv; Array_: JShortArray; Elems: PJShort; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseIntArrayElements: procedure(Env: PJNIEnv; Array_: JIntArray; Elems: PJInt; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseLongArrayElements: procedure(Env: PJNIEnv; Array_: JLongArray; Elems: PJLong; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseFloatArrayElements: procedure(Env: PJNIEnv; Array_: JFloatArray; Elems: PJFloat; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseDoubleArrayElements: procedure(Env: PJNIEnv; Array_: JDoubleArray; Elems: PJDouble; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetBooleanArrayRegion: procedure(Env: PJNIEnv; Array_: JBooleanArray; Start, Len: JSize; Buf: PJBoolean); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetByteArrayRegion: procedure(Env: PJNIEnv; Array_: JByteArray; Start, Len: JSize; Buf: PJByte); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetCharArrayRegion: procedure(Env: PJNIEnv; Array_: JCharArray; Start, Len: JSize; Buf: PJChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetShortArrayRegion: procedure(Env: PJNIEnv; Array_: JShortArray; Start, Len: JSize; Buf: PJShort); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetIntArrayRegion: procedure(Env: PJNIEnv; Array_: JIntArray; Start, Len: JSize; Buf: PJInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetLongArrayRegion: procedure(Env: PJNIEnv; Array_: JLongArray; Start, Len: JSize; Buf: PJLong); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetFloatArrayRegion: procedure(Env: PJNIEnv; Array_: JFloatArray; Start, Len: JSize; Buf: PJFloat); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetDoubleArrayRegion: procedure(Env: PJNIEnv; Array_: JDoubleArray; Start, Len: JSize; Buf: PJDouble); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    SetBooleanArrayRegion: procedure(Env: PJNIEnv; Array_: JBooleanArray; Start, Len: JSize; const Buf: PJBoolean); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetByteArrayRegion: procedure(Env: PJNIEnv; Array_: JByteArray; Start, Len: JSize; const Buf: PJByte); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetCharArrayRegion: procedure(Env: PJNIEnv; Array_: JCharArray; Start, Len: JSize; const Buf: PJChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetShortArrayRegion: procedure(Env: PJNIEnv; Array_: JShortArray; Start, Len: JSize; const Buf: PJShort); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetIntArrayRegion: procedure(Env: PJNIEnv; Array_: JIntArray; Start, Len: JSize; const Buf: PJInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetLongArrayRegion: procedure(Env: PJNIEnv; Array_: JLongArray; Start, Len: JSize; const Buf: PJLong); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetFloatArrayRegion: procedure(Env: PJNIEnv; Array_: JFloatArray; Start, Len: JSize; const Buf: PJFloat); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    SetDoubleArrayRegion: procedure(Env: PJNIEnv; Array_: JDoubleArray; Start, Len: JSize; const Buf: PJDouble); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    RegisterNatives: function(Env: PJNIEnv; Clazz: JClass; const Methods: PJNINativeMethod; NMethods: JInt): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    UnregisterNatives: function(Env: PJNIEnv; Clazz: JClass): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    MonitorEnter: function(Env: PJNIEnv; Obj: JObject): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    MonitorExit: function(Env: PJNIEnv; Obj: JObject): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetJavaVM: function(Env: PJNIEnv; out VM: PJavaVM): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetStringRegion: procedure(Env: PJNIEnv; Str: JString; Start, Len: JSize; Buf: PJChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetStringUTFRegion: procedure(Env: PJNIEnv; Str: JString; Start, Len: JSize; Buf: PAnsiChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetPrimitiveArrayCritical: function(Env: PJNIEnv; Array_: JArray; var IsCopy: JBoolean): Pointer; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleasePrimitiveArrayCritical: procedure(Env: PJNIEnv; Array_: JArray; Carray: Pointer; Mode: JInt); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    GetStringCritical: function(Env: PJNIEnv; String_: JString; var IsCopy: JBoolean): PJChar; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    ReleaseStringCritical: procedure(Env: PJNIEnv; String_: JString; const CString: PJChar); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    NewWeakGlobalRef: function(Env: PJNIEnv; Obj: JObject): JWeak; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    DeleteWeakGlobalRef: procedure(Env: PJNIEnv; Ref: JWeak); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    ExceptionCheck: function(Env: PJNIEnv): JBoolean; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    NewDirectByteBuffer: function(Env: PJNIEnv; Address: Pointer; Capacity: JLong): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetDirectBufferAddress: function(Env: PJNIEnv; Buf: JObject): Pointer; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetDirectBufferCapacity: function(Env: PJNIEnv; Buf: JObject): JLong; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    (* New JNI 1.6 Features *)

    GetObjectRefType: function(Env: PJNIEnv; Obj: JObject): JObjectRefType; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

    (* Module Features *)

    GetModule: function(Env: PJNIEnv; Clazz: JClass): JObject; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
  end;

(*
 * optionString may be any option accepted by the JVM, or one of the
 * following:
 *
 * -D<name>=<value>          Set a system property.
 * -verbose[:class|gc|jni]   Enable verbose output, comma-separated. E.g.
 *                           "-verbose:class" or "-verbose:gc,class"
 *                           Standard names include: gc, class, and jni.
 *                           All nonstandard (VM-specific) names must begin
 *                           with "X".
 * vfprintf                  extraInfo is a pointer to the vfprintf hook.
 * exit                      extraInfo is a pointer to the exit hook.
 * abort                     extraInfo is a pointer to the abort hook.
 *)

  JavaVMOption = packed record
    OptionString: PAnsiChar;
    ExtraInfo: Pointer;
  end;

  JavaVMInitArgs = packed record
    Version: JInt;
    NOptions: JInt;
    Options: ^JavaVMOption;
    IgnoreUnrecognized: JBoolean;
  end;

  JavaVMAttachArgs = packed record
    Version: JInt;
    Name: PAnsiChar;
    Group: JObject;
  end;

  JNIInvokeInterface_ = packed record
    reserved0: Pointer;
    reserved1: Pointer;
    reserved2: Pointer;

    DestroyJavaVM: function(Vm: PJavaVM): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    AttachCurrentThread: function(Vm: PJavaVM; out Env: PJNIEnv; var Args: JavaVMAttachArgs): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    DetachCurrentThread: function(Vm: PJavaVM): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    GetEnv: function(Vm: PJavaVM; out Env: Pointer; Version: JInt): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
    AttachCurrentThreadAsDaemon: function(Vm: PJavaVM; out Env: PJNIEnv; var Args: JavaVMAttachArgs): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
  end;

  JNI_GetDefaultJavaVMInitArgs = function(out Args: JavaVMInitArgs): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
  JNI_CreateJavaVM = function(out VM: PJavaVM; out Env: PJNIEnv; var Args: JavaVMInitArgs): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
  JNI_GetCreatedJavaVMs = function(out VM: PJavaVM; Size: JSize; out OutSize: JSize): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

  // Defined by native libraries
  JNI_OnLoad = function(VM: PJavaVM; Reserved: Pointer): JInt; {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}
  JNI_OnUnload = procedure(VM: PJavaVM; Reserved: Pointer); {$ifdef MSWINDOWS} stdcall; {$ELSE} cdecl; {$endif MSWINDOWS}

const
  JNI_VERSION_1_1 = $00010001;
  JNI_VERSION_1_2 = $00010002;
  JNI_VERSION_1_4 = $00010004;
  JNI_VERSION_1_6 = $00010006;
  JNI_VERSION_1_8 = $00010008;
  JNI_VERSION_9   = $00090000;
  JNI_VERSION_10  = $000a0000;

implementation

end.